# coding: utf-8

"""
    Copyright (c) 2021-2022 VMware, Inc. All rights reserved.
    VMware Confidential
    Author: dpallagolla@vmware.com
"""

from __future__ import absolute_import

import ast
import unittest

from api.client.sdk_manager import SDKManager
from api.client.session_manager import HybriditySessionManager
from caas_v2 import ClusterSpec, ClusterSpecClusterConfigSpec, ClusterSpecControlPlaneSpec, \
    ClusterSpecClusterConfigSpecClusterNetwork, ClusterSpecClusterConfigSpecClusterNetworkPods, \
    ClusterSpecClusterConfigSpecClusterNetworkServices, ClusterSpecClusterConfigSpecControlPlaneEndpoint, \
    ClusterSpecClusterConfigSpecStrategyRollingUpdate, ClusterSpecClusterConfigSpecStrategy, \
    ClusterSpecClusterConfigSpecTcaBomReleaseRef, ClusterSpecClusterConfigSpecCloudProviders, \
    ClusterSpecClusterConfigSpecCloudProvidersPrimeRef, ClusterSpecControlPlaneSpecCloudProvider, \
    ClusterSpecControlPlaneSpecCloudMachineTemplate, ClusterSpecControlPlaneSpecCloudMachineTemplateVSphereMachine, \
    ClusterSpecControlPlaneSpecNetwork, ClusterSpecControlPlaneSpecNetworkDevices, Cluster
from cluster import ClustersQueryBody
from tca.api.client.tca_configuration import TCAConfiguration  # VMW005


class TestClusterAutomationApi(unittest.TestCase):

    def setUp(self):
        tca_config = TCAConfiguration()
        tca_config.set_server('https://tca_server')      # Ensure no trailing slash '/' in the tca_server
        tca_config.set_username('tenant@vsphere.local')
        tca_config.set_password('********************')
        tca_config.set_verify_ssl(False)                 # Insecure: only for NFV functional validation

        hybridity_sessionmgr = HybriditySessionManager()
        self.sdk_manager = SDKManager(). \
            with_configuration(tca_config). \
            with_session_manager(hybridity_sessionmgr)

        self.cluster_automation_api_client = self.sdk_manager.get_api('tca.caas_v2:ClusterAutomationApi')
        self.generic_cr_api = self.sdk_manager.get_api('tca.caas_k8s_generic_cr:GenericCrApi')
        self.cluster_client = self.sdk_manager.get_api('tca.cluster:ClusterApi')

    def tearDown(self):
        pass

    def test_create_cluster(self):
        # vim_id of the TCA-CP where the cluster has to be deployed
        vim_id = ''
        # data center where the cluster has to be deployed
        data_center = ''
        kubernetes_version_for_cluster = 'v1.20.15'
        mgmt_cluster_name = ''

        cluster_config_spec = self.build_cluster_config_spec(mgmt_cluster_name,
                                                             kubernetes_version_for_cluster, vim_id, data_center)
        control_plane_spec = self.build_control_plane_spec(cluster_config_spec, vim_id, data_center)
        cluster_spec = ClusterSpec(cluster_config_spec=cluster_config_spec, control_plane_spec=control_plane_spec)
        cluster_payload = Cluster(spec=cluster_spec)
        self.cluster_automation_api_client.create_cluster(cluster_payload, mgmt_cluster_name=mgmt_cluster_name,
                                                          tca_cp_id=self.tca_cp_id)

    def build_control_plane_spec(self, cluster_config_spec, vim_id, data_center):
        clone_mode = ''
        control_plane_name = 'test-cluster-control-plane'
        cloud_provider = ClusterSpecControlPlaneSpecCloudProvider(vim_id=vim_id)
        cluster_name = 'test-cluster'
        disk_gib = 30
        tca_bom_release_ref = cluster_config_spec.tca_bom_release_ref
        memory_mi_b = 8192
        num_cpus = 8
        vm_template_path_for_kubernetes_version = ''
        data_store = ''
        resource_pool = ''
        folder = ''
        nameservers = []
        network_name = ''

        devices = ClusterSpecControlPlaneSpecNetworkDevices(dhcp4=True, network_name=network_name,
                                                            nameservers=nameservers)
        network = ClusterSpecControlPlaneSpecNetwork(devices=[devices])

        vsphere_machine = ClusterSpecControlPlaneSpecCloudMachineTemplateVSphereMachine(datacenter=data_center,
                                                                                        datastore=data_store,
                                                                                        resource_pool=resource_pool,
                                                                                        template=vm_template_path_for_kubernetes_version,
                                                                                        folder=folder)

        cloud_machine_template = ClusterSpecControlPlaneSpecCloudMachineTemplate(v_sphere_machine=vsphere_machine)
        return ClusterSpecControlPlaneSpec(clone_mode=clone_mode, control_plane_name=control_plane_name,
                                           cloud_provider=cloud_provider, cluster_name=cluster_name,
                                           disk_gi_b=disk_gib, tca_bom_release_ref=tca_bom_release_ref,
                                           memory_mi_b=memory_mi_b, num_cp_us=num_cpus, network=network,
                                           cloud_machine_template=cloud_machine_template)

    def build_cluster_config_spec(self, mgmt_cluster_name, kubernetes_version_for_cluster, vim_id, data_center):
        host_ip = ''
        cluster_network = self.build_cluster_network()
        control_plane_endpoint = ClusterSpecClusterConfigSpecControlPlaneEndpoint(host=host_ip)
        strategy = self.get_cluster_config_spec_strategy()
        tca_bom_release_ref = self.build_tca_bom_release_ref(mgmt_cluster_name=mgmt_cluster_name,
                                                             kubernetes_version_for_cluster=kubernetes_version_for_cluster)
        cloud_providers = self.build_cloud_providers_ref(vim_id, data_center)

        return ClusterSpecClusterConfigSpec(cluster_network=cluster_network,
                                            control_plane_endpoint=control_plane_endpoint,
                                            cni_type='antrea', strategy=strategy,
                                            tca_bom_release_ref=tca_bom_release_ref,
                                            cloud_providers=cloud_providers)

    def build_cloud_providers_ref(self, vim_id, data_center):
        prime_ref = ClusterSpecClusterConfigSpecCloudProvidersPrimeRef(vim_id=vim_id, datacenter=data_center)
        return ClusterSpecClusterConfigSpecCloudProviders(prime_ref=prime_ref)

    def build_tca_bom_release_ref(self, mgmt_cluster_name, kubernetes_version_for_cluster):

        mgmt_cluster_filter = ClustersQueryBody(filter={"clusterName": mgmt_cluster_name})
        cluster_infra_cluster_query_response = self.cluster_client.infra_cluster_query(body=mgmt_cluster_filter)

        mgmt_cluster = cluster_infra_cluster_query_response[0]
        mgmt_cluster_name = mgmt_cluster.cluster_name
        tca_cp_id = self.get_tca_cp_id_for_mgmt_cluster(mgmt_cluster_name=mgmt_cluster_name)
        self.tca_cp_id = tca_cp_id

        tca_bom_list = self.generic_cr_api.list_tca_bom_releases(tca_cp_id=tca_cp_id,
                                                                 mgmt_cluster_name=mgmt_cluster_name)
        items = ast.literal_eval(tca_bom_list)["items"]
        tca_bom_ref = None
        for item in items:
            if kubernetes_version_for_cluster in item['metadata']['name']:
                tca_bom_ref = item
                break
        tca_bom_ref_name = tca_bom_ref['metadata']['name']

        return ClusterSpecClusterConfigSpecTcaBomReleaseRef(name=tca_bom_ref_name)

    def get_cluster_config_spec_strategy(self):
        rolling_update = ClusterSpecClusterConfigSpecStrategyRollingUpdate(max_surge=1, max_unavailable=0)
        return ClusterSpecClusterConfigSpecStrategy(type='RollingUpdate', rolling_update=rolling_update)

    def build_cluster_network(self):
        pods = ClusterSpecClusterConfigSpecClusterNetworkPods(cidr_blocks=['100.96.0.0/11'])
        services = ClusterSpecClusterConfigSpecClusterNetworkServices(cidr_blocks=['100.64.0.0/13'])
        return ClusterSpecClusterConfigSpecClusterNetwork(pods=pods, services=services)

    def get_tca_cp_id_for_mgmt_cluster(self, mgmt_cluster_name):
        mgmt_cluster_filter = ClustersQueryBody(filter={"clusterName": mgmt_cluster_name})
        cluster_infra_cluster_query_response = self.cluster_client.infra_cluster_query(body=mgmt_cluster_filter)
        mgmt_cluster = cluster_infra_cluster_query_response[0]
        return mgmt_cluster.hcx_uuid

    def test_delete_cluster(self):
        """Test case for delete_cluster

        Api to delete kubernetes Cluster for Telco Cloud Automation  # noqa: E501
        """
        pass

    def test_edit_cluster(self):
        workload_cluster_name = ''
        desired_kubernetes_version_for_cluster = 'v1.21.11'
        vm_template_path_for_desired_kubernetes_version = ''
        mgmt_cluster_name = ''
        desired_bom_release_ref = self.build_tca_bom_release_ref(mgmt_cluster_name=mgmt_cluster_name,
                                                                 kubernetes_version_for_cluster=desired_kubernetes_version_for_cluster)
        tca_cp_id = self.get_tca_cp_id_for_mgmt_cluster(mgmt_cluster_name=mgmt_cluster_name)
        current_cluster = self.cluster_automation_api_client.get_cluster(cluster_name=workload_cluster_name,
                                                                         mgmt_cluster_name=mgmt_cluster_name,
                                                                         tca_cp_id=tca_cp_id)
        current_cluster.spec.cluster_config_spec.tca_bom_release_ref = desired_bom_release_ref
        current_cluster.spec.control_plane_spec.cloud_machine_template.v_sphere_machine.template = vm_template_path_for_desired_kubernetes_version
        current_cluster.spec.control_plane_spec.tca_bom_release_ref = desired_bom_release_ref
        self.cluster_automation_api_client.edit_cluster(current_cluster, cluster_name=workload_cluster_name,
                                                        mgmt_cluster_name=mgmt_cluster_name,
                                                        tca_cp_id=tca_cp_id)

    def test_get_addon_tca_k8s_resources(self):
        """Test case for get_addon_tca_k8s_resources

        API to fetch underyling tca k8s CRs (Deployed by cluster lifecycle operator) including TcaNodePool & NodePolicy  # noqa: E501
        """
        pass

    def test_get_all_clusters(self):
        data = self.cluster_automation_api_client.get_all_clusters(async_req=False)
        print("get all clusters response is", data)

    def test_get_cluster(self):
        workload_cluster_name = 'workload-v2'
        mgmt_cluster_name = 'mgmt-cluster'
        tca_cp_id = self.get_tca_cp_id_for_mgmt_cluster(mgmt_cluster_name=mgmt_cluster_name)
        data = self.cluster_automation_api_client.get_cluster(cluster_name=workload_cluster_name,
                                                              mgmt_cluster_name=mgmt_cluster_name,
                                                              tca_cp_id=tca_cp_id, async_req=False)
        print("get cluster response is", data)

    def test_get_cluster_stats(self):
        """Test case for get_cluster_stats

        Api to get kubernetes Cluster Stats for Telco Cloud Automation  # noqa: E501
        """
        pass

    def test_get_cluster_tca_k8s_resources(self):
        """Test case for get_cluster_tca_k8s_resources

        API to fetch underyling tca k8s CRs (Deployed by cluster lifecycle operator) including TCAKubernetesCluster & TCAKubeControlPlane  # noqa: E501
        """
        pass

    def test_get_node_pool_tca_k8s_resources(self):
        """Test case for get_node_pool_tca_k8s_resources

        API to fetch underyling tca k8s CRs (Deployed by cluster lifecycle operator) including TcaNodePool & NodePolicy  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()
